@@ -3,25 +3,31 @@ This plotting module provides various functions to plot the data measured
 using QCoDeS.
 """
 
-import logging
-from functools import partial
-from typing import (Optional, List, Sequence, Union, Tuple, Dict,
-                    Any, Set, cast)
 import inspect
-import numpy as np
+import logging
+from contextlib import contextmanager
+from functools import partial
+from typing import Any, List, Optional, Sequence, Tuple, Union, cast
+
 import matplotlib
 import matplotlib.pyplot as plt
+import numpy as np
 from matplotlib.ticker import FuncFormatter
-from contextlib import contextmanager
+from typing_extensions import Literal
 
 import qcodes as qc
-from qcodes.dataset.data_set import load_by_run_spec, DataSet
-from qcodes.utils.plotting import (auto_color_scale_from_config,
-                                   find_scale_and_prefix)
+from qcodes.dataset.data_set import load_by_run_spec
+from qcodes.dataset.data_set_protocol import DataSetProtocol
+from qcodes.utils.plotting import auto_color_scale_from_config, find_scale_and_prefix
 
-from .data_export import (get_data_by_id, flatten_1D_data_for_plot,
-                          get_1D_plottype, get_2D_plottype, reshape_2D_data,
-                          _strings_as_ints)
+from .data_export import (
+    DSPlotData,
+    _get_data_from_ds,
+    _strings_as_ints,
+    get_1D_plottype,
+    get_2D_plottype,
+    reshape_2D_data,
+)
 
 log = logging.getLogger(__name__)
 DB = qc.config["core"]["db_location"]
@@ -29,10 +35,9 @@ DB = qc.config["core"]["db_location"]
 AxesTuple = Tuple[matplotlib.axes.Axes, matplotlib.colorbar.Colorbar]
 AxesTupleList = Tuple[List[matplotlib.axes.Axes],
                       List[Optional[matplotlib.colorbar.Colorbar]]]
-Number = Union[float, int]
-# NamedData is the structure get_data_by_id returns and that plot_by_id
+# NamedData is the structure _get_data_from_ds returns and that plot_by_id
 # uses internally
-NamedData = List[List[Dict[str, Union[str, np.ndarray]]]]
+NamedData = List[List[DSPlotData]]
 
 # list of kwargs for plotting function, so that kwargs can be passed to
 # :func:`plot_dataset` and will be distributed to the respective plotting func.
@@ -86,19 +91,19 @@ def _appropriate_kwargs(plottype: str,
     yield plot_handler_mapping[plottype](**kwargs.copy())
 
 
-def plot_dataset(dataset: DataSet,
-                 axes: Optional[Union[matplotlib.axes.Axes,
-                                      Sequence[matplotlib.axes.Axes]]] = None,
-                 colorbars: Optional[Union[matplotlib.colorbar.Colorbar,
-                                           Sequence[
-                                        matplotlib.colorbar.Colorbar]]] = None,
-                 rescale_axes: bool = True,
-                 auto_color_scale: Optional[bool] = None,
-                 cutoff_percentile: Optional[Union[Tuple[Number, Number],
-                                                   Number]] = None,
-                 complex_plot_type: str = 'real_and_imag',
-                 complex_plot_phase: str = 'radians',
-                 **kwargs: Any) -> AxesTupleList:
+def plot_dataset(
+    dataset: DataSetProtocol,
+    axes: Optional[Union[matplotlib.axes.Axes, Sequence[matplotlib.axes.Axes]]] = None,
+    colorbars: Optional[
+        Union[matplotlib.colorbar.Colorbar, Sequence[matplotlib.colorbar.Colorbar]]
+    ] = None,
+    rescale_axes: bool = True,
+    auto_color_scale: Optional[bool] = None,
+    cutoff_percentile: Optional[Union[Tuple[float, float], float]] = None,
+    complex_plot_type: Literal["real_and_imag", "mag_and_phase"] = "real_and_imag",
+    complex_plot_phase: Literal["radians", "degrees"] = "radians",
+    **kwargs: Any,
+) -> AxesTupleList:
     """
     Construct all plots for a given dataset
 
@@ -176,7 +181,7 @@ def plot_dataset(dataset: DataSet,
     title = f"Run #{dataset.captured_run_id}, " \
             f"Experiment {experiment_name} ({sample_name})"
 
-    alldata: NamedData = get_data_by_id(dataset.run_id)
+    alldata: NamedData = _get_data_from_ds(dataset)
     alldata = _complex_to_real_preparser(alldata,
                                          conversion=complex_plot_type,
                                          degrees=degrees)
@@ -214,8 +219,8 @@ def plot_dataset(dataset: DataSet,
         if len(data) == 2:  # 1D PLOTTING
             log.debug(f'Doing a 1D plot with kwargs: {kwargs}')
 
-            xpoints = cast(np.ndarray, data[0]['data'])
-            ypoints = cast(np.ndarray, data[1]['data'])
+            xpoints = data[0]["data"]
+            ypoints = data[1]["data"]
 
             plottype = get_1D_plottype(xpoints, ypoints)
             log.debug(f'Determined plottype: {plottype}')
@@ -252,17 +257,17 @@ def plot_dataset(dataset: DataSet,
         elif len(data) == 3:  # 2D PLOTTING
             log.debug(f'Doing a 2D plot with kwargs: {kwargs}')
 
-            # From the setpoints, figure out which 2D plotter to use
-            # TODO: The "decision tree" for what gets plotted how and how
-            # we check for that is still unfinished/not optimised
-
-            xpoints = flatten_1D_data_for_plot(data[0]['data'])
-            ypoints = flatten_1D_data_for_plot(data[1]['data'])
-            zpoints = flatten_1D_data_for_plot(data[2]['data'])
-
-            plottype = get_2D_plottype(xpoints, ypoints, zpoints)
-
-            log.debug(f'Determined plottype: {plottype}')
+            if data[2]["shape"] is None:
+                xpoints = data[0]["data"].flatten()
+                ypoints = data[1]["data"].flatten()
+                zpoints = data[2]["data"].flatten()
+                plottype = get_2D_plottype(xpoints, ypoints, zpoints)
+                log.debug(f"Determined plottype: {plottype}")
+            else:
+                xpoints = data[0]["data"]
+                ypoints = data[1]["data"]
+                zpoints = data[2]["data"]
+                plottype = "2D_grid"
 
             how_to_plot = {'2D_grid': plot_on_a_plain_grid,
                            '2D_equidistant': plot_on_a_plain_grid,
@@ -301,23 +306,23 @@ def plot_dataset(dataset: DataSet,
     return axeslist, new_colorbars
 
 
-def plot_by_id(run_id: int,
-               axes: Optional[Union[matplotlib.axes.Axes,
-                              Sequence[matplotlib.axes.Axes]]] = None,
-               colorbars: Optional[Union[matplotlib.colorbar.Colorbar,
-                                   Sequence[
-                                       matplotlib.colorbar.Colorbar]]] = None,
-               rescale_axes: bool = True,
-               auto_color_scale: Optional[bool] = None,
-               cutoff_percentile: Optional[Union[Tuple[Number, Number],
-                                                 Number]] = None,
-               complex_plot_type: str = 'real_and_imag',
-               complex_plot_phase: str = 'radians',
-               **kwargs: Any) -> AxesTupleList:
+def plot_by_id(
+    run_id: int,
+    axes: Optional[Union[matplotlib.axes.Axes, Sequence[matplotlib.axes.Axes]]] = None,
+    colorbars: Optional[
+        Union[matplotlib.colorbar.Colorbar, Sequence[matplotlib.colorbar.Colorbar]]
+    ] = None,
+    rescale_axes: bool = True,
+    auto_color_scale: Optional[bool] = None,
+    cutoff_percentile: Optional[Union[Tuple[float, float], float]] = None,
+    complex_plot_type: Literal["real_and_imag", "mag_and_phase"] = "real_and_imag",
+    complex_plot_phase: Literal["radians", "degrees"] = "radians",
+    **kwargs: Any,
+) -> AxesTupleList:
     """
     Construct all plots for a given `run_id`. Here `run_id` is an
     alias for `captured_run_id` for historical reasons. See the docs
-    of :func:`.load_by_run_spec` for details of loading runs.
+    of :func:`qcodes.dataset.load_by_run_spec` for details of loading runs.
     All other arguments are forwarded
     to :func:`.plot_dataset`, see this for more details.
     """
@@ -334,15 +339,17 @@ def plot_by_id(run_id: int,
                         **kwargs)
 
 
-def _complex_to_real_preparser(alldata: NamedData,
-                               conversion: str,
-                               degrees: bool=False) -> NamedData:
+def _complex_to_real_preparser(
+    alldata: Sequence[Sequence[DSPlotData]],
+    conversion: Literal["real_and_imag", "mag_and_phase"],
+    degrees: bool = False,
+) -> NamedData:
     """
     Convert complex-valued parameters to two real-valued parameters, either
     real and imaginary part or phase and magnitude part
 
     Args:
-        alldata: The data to convert, should be the output of get_data_by_id
+        alldata: The data to convert, should be the output of `_get_data_from_ds`
         conversion: the conversion method, either "real_and_imag" or
             "mag_and_phase"
         degrees: Whether to return the phase in degrees. The default is to
@@ -354,7 +361,7 @@ def _complex_to_real_preparser(alldata: NamedData,
                          'but can only accept "real_and_imag" or '
                          '"mag_and_phase".')
 
-    newdata = []
+    newdata: NamedData = []
 
     # we build a new NamedData object from the given `alldata` input.
     # Note that the length of `newdata` will be larger than that of `alldata`
@@ -367,11 +374,11 @@ def _complex_to_real_preparser(alldata: NamedData,
         new_group = []
         new_groups: NamedData = [[], []]
         for index, parameter in enumerate(group):
-            data = cast(np.ndarray, parameter['data'])
-            if data.dtype.kind == 'c':
-                p1, p2 = _convert_complex_to_real(parameter,
-                                                  conversion=conversion,
-                                                  degrees=degrees)
+            data = parameter["data"]
+            if data.dtype.kind == "c":
+                p1, p2 = _convert_complex_to_real(
+                    parameter, conversion=conversion, degrees=degrees
+                )
                 if index < len(group) - 1:
                     # if the above condition is met, we are dealing with
                     # complex setpoints
@@ -401,11 +408,8 @@ def _complex_to_real_preparser(alldata: NamedData,
 
 
 def _convert_complex_to_real(
-        parameter: Dict[str, Union[str, np.ndarray]],
-        conversion: str,
-        degrees: bool
-        ) -> Tuple[Dict[str, Union[str, np.ndarray]],
-                   Dict[str, Union[str, np.ndarray]]]:
+    parameter: DSPlotData, conversion: str, degrees: bool
+) -> Tuple[DSPlotData, DSPlotData]:
     """
     Do the actual conversion and turn one parameter into two.
     Should only be called from within _complex_to_real_preparser.
@@ -429,20 +433,26 @@ def _convert_complex_to_real(
     new_units = converters['units'][conversion](parameter['unit'])
     new_names = converters['names'][conversion](parameter['name'])
 
-    new_parameters = tuple(
-        {'name': name, 'label': label,
-         'unit': unit, 'data': data}
-        for name, label, unit, data in zip(
-            new_names, new_labels, new_units, new_data))
+    parameter1: DSPlotData = {
+        "name": new_names[0],
+        "label": new_labels[0],
+        "unit": new_units[0],
+        "data": new_data[0],
+        "shape": parameter["shape"],
+    }
 
-    # The reason we ignore the type in the return is that I cannot figure
-    # out how to get mypy to correctly infer the type of iterated values
-    # (the name, label, unit, and data above)
+    parameter2: DSPlotData = {
+        "name": new_names[1],
+        "label": new_labels[1],
+        "unit": new_units[1],
+        "data": new_data[1],
+        "shape": parameter["shape"],
+    }
 
-    return new_parameters  # type: ignore[return-value]
+    return parameter1, parameter2
 
 
-def _get_label_of_data(data_dict: Dict[str, Any]) -> str:
+def _get_label_of_data(data_dict: DSPlotData) -> str:
     return data_dict['label'] if data_dict['label'] != '' \
         else data_dict['name']
 
@@ -454,17 +464,17 @@ def _make_axis_label(label: str, unit: str) -> str:
     return label
 
 
-def _make_label_for_data_axis(data: List[Dict[str, Any]], axis_index: int
-                              ) -> str:
+def _make_label_for_data_axis(data: Sequence[DSPlotData], axis_index: int) -> str:
     label = _get_label_of_data(data[axis_index])
     unit = data[axis_index]['unit']
     return _make_axis_label(label, unit)
 
 
-def _set_data_axes_labels(ax: matplotlib.axes.Axes,
-                          data: List[Dict[str, Any]],
-                          cax: Optional[matplotlib.colorbar.Colorbar] = None
-                          ) -> None:
+def _set_data_axes_labels(
+    ax: matplotlib.axes.Axes,
+    data: Sequence[DSPlotData],
+    cax: Optional[matplotlib.colorbar.Colorbar] = None,
+) -> None:
     ax.set_xlabel(_make_label_for_data_axis(data, 0))
     ax.set_ylabel(_make_label_for_data_axis(data, 1))
 
@@ -574,24 +584,24 @@ def plot_on_a_plain_grid(x: np.ndarray,
         z_strings = np.unique(z)
         z = _strings_as_ints(z)
 
-    xrow, yrow, z_to_plot = reshape_2D_data(x, y, z)
+    if x.ndim == 2 and y.ndim == 2 and z.ndim == 2:
+        if not np.logical_or(np.any(np.isnan(x)), np.any(np.isnan(y))):
+            # data is on a grid that may or may not be
+            # rectilinear. Rely on matplotlib to plot
+            # this directly
+            x_to_plot, y_to_plot, z_to_plot = x, y, z
+            num_points = x_to_plot.size
+        else:
+            x_to_plot, y_to_plot, z_to_plot = _clip_nan_from_shaped_data(x, y, z)
+            num_points = x_to_plot.size * y_to_plot.size
+    else:
+        x_to_plot, y_to_plot, z_to_plot = reshape_2D_data(x, y, z)
+        num_points = x_to_plot.size * y_to_plot.size
 
-    # we use a general edge calculator,
-    # in the case of non-equidistantly spaced data
-    # TODO: is this appropriate for a log ax?
-    dxs = np.diff(xrow)/2
-    dys = np.diff(yrow)/2
-    x_edges = np.concatenate((np.array([xrow[0] - dxs[0]]),
-                              xrow[:-1] + dxs,
-                              np.array([xrow[-1] + dxs[-1]])))
-    y_edges = np.concatenate((np.array([yrow[0] - dys[0]]),
-                              yrow[:-1] + dys,
-                              np.array([yrow[-1] + dys[-1]])))
     if 'rasterized' in kwargs.keys():
         rasterized = kwargs.pop('rasterized')
     else:
-        rasterized = len(x_edges) * len(y_edges) \
-                      > qc.config.plotting.rasterize_threshold
+        rasterized = num_points > qc.config.plotting.rasterize_threshold
 
     cmap = kwargs.pop('cmap') if 'cmap' in kwargs else None
 
@@ -599,11 +609,15 @@ def plot_on_a_plain_grid(x: np.ndarray,
         name = cmap.name if hasattr(cmap, 'name') else 'viridis'
         cmap = matplotlib.cm.get_cmap(name, len(z_strings))
 
-    colormesh = ax.pcolormesh(x_edges, y_edges,
-                              np.ma.masked_invalid(z_to_plot),
-                              rasterized=rasterized,
-                              cmap=cmap,
-                              **kwargs)
+    colormesh = ax.pcolormesh(
+        x_to_plot,
+        y_to_plot,
+        np.ma.masked_invalid(z_to_plot),
+        rasterized=rasterized,
+        cmap=cmap,
+        shading="nearest",
+        **kwargs,
+    )
 
     if x_is_stringy:
         ax.set_xticks(np.arange(len(np.unique(x_strings))))
@@ -627,16 +641,61 @@ def plot_on_a_plain_grid(x: np.ndarray,
     return ax, colorbar
 
 
+def _clip_nan_from_shaped_data(
+    x: np.ndarray, y: np.ndarray, z: np.ndarray
+) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    def _on_rectilinear_grid_except_nan(x_data: np.ndarray, y_data: np.ndarray) -> bool:
+        """
+        check that data is on a rectilinear grid. e.g. all points are the same as the  first
+        row and column with the exception of nans. Those represent points not yet measured.
+        """
+        x_row = x_data[:, 0:1]
+        y_row = y_data[0:1, :]
+        return (
+            np.nanmax(np.abs(x_data - x_row)) == 0
+            and np.nanmax(np.abs(y_data - y_row)) == 0
+        )
+
+    if _on_rectilinear_grid_except_nan(x, y):
+        # clip any row or column where there are nans in the first row
+        # or column. Since we fill from here we assume that means that no data
+        # has been measured for this row
+        x_to_plot, y_to_plot = x[:, 0], y[0, :]
+
+        filter_x = ~np.isnan(x_to_plot)
+        filter_y = ~np.isnan(y_to_plot)
+
+        x_to_plot = x_to_plot[filter_x]
+        y_to_plot = y_to_plot[filter_y]
+        z_to_plot = z[filter_x, :]
+        z_to_plot = z_to_plot[:, filter_y].transpose()
+    else:
+        # fallback to flattening the data and use the same path as
+        # non shaped data after filtering the nans.
+        # this is not ideal as we loose the shape data but
+        # not clear how to do this better. Either return a ragged
+        # array or clip all inner dims that have nans completely
+        x = x.flatten()
+        y = y.flatten()
+        z = z.flatten()
+        filter_nans = np.logical_and(~np.isnan(x), ~np.isnan(y))
+        x_to_plot, y_to_plot, z_to_plot = reshape_2D_data(
+            x[filter_nans], y[filter_nans], z[filter_nans]
+        )
+    return x_to_plot, y_to_plot, z_to_plot
+
+
 def _scale_formatter(tick_value: float, pos: int, factor: float) -> str:
     """
     Function for matplotlib.ticker.FuncFormatter that scales the tick values
     according to the given `scale` value.
     """
-    return "{:g}".format(tick_value*factor)
+    return f"{tick_value*factor:g}"
 
 
-def _make_rescaled_ticks_and_units(data_dict: Dict[str, Any]) \
-        -> Tuple[matplotlib.ticker.FuncFormatter, str]:
+def _make_rescaled_ticks_and_units(
+    data_dict: DSPlotData,
+) -> Tuple[matplotlib.ticker.FuncFormatter, str]:
     """
     Create a ticks formatter and a new label for the data that is to be used
     on the axes where the data is plotted.
@@ -680,10 +739,11 @@ def _make_rescaled_ticks_and_units(data_dict: Dict[str, Any]) \
     return ticks_formatter, new_label
 
 
-def _rescale_ticks_and_units(ax: matplotlib.axes.Axes,
-                             data: List[Dict[str, Any]],
-                             cax: matplotlib.colorbar.Colorbar = None
-                             ) -> None:
+def _rescale_ticks_and_units(
+    ax: matplotlib.axes.Axes,
+    data: Sequence[DSPlotData],
+    cax: matplotlib.colorbar.Colorbar = None,
+) -> None:
     """
     Rescale ticks and units for the provided axes as described in
     :func:`~_make_rescaled_ticks_and_units`
