@@ -5,20 +5,31 @@ This module provides infrastructure for wrapping DLL libraries, loaded using
 from the DLL library with mostly python types in mind, and conveniently
 specify their signatures in terms of :mod:`ctypes` types.
 """
+from __future__ import annotations
 
+import concurrent
+import concurrent.futures
 import ctypes
 import logging
-from typing import Type, Dict, NamedTuple, Sequence, NewType, List, Any, TypeVar, Callable, Tuple
-from threading import Lock
-import concurrent
 from functools import partial
+from threading import Lock
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    NewType,
+    Sequence,
+    Tuple,
+    TypeVar,
+)
 from weakref import WeakValueDictionary
 
 from qcodes.instrument.parameter import _BaseParameter
-from .utils import TraceParameter
-from .constants import API_SUCCESS, API_DMA_IN_PROGRESS, ERROR_CODES, \
-    ReturnCode
 
+from .constants import API_DMA_IN_PROGRESS, API_SUCCESS, ERROR_CODES, ReturnCode
+from .utils import TraceParameter
 
 logger = logging.getLogger(__name__)
 
@@ -59,7 +70,7 @@ def _mark_params_as_updated(*args: Any) -> None:
 
 def _check_error_code(
         return_code: int,
-        func: Callable,
+        func: Callable[..., Any],
         arguments: Tuple[Any, ...]
 ) -> Tuple[Any, ...]:
     if (return_code != API_SUCCESS) and (return_code != API_DMA_IN_PROGRESS):
@@ -86,7 +97,7 @@ def _check_error_code(
 
 def _convert_bytes_to_str(
         output: bytes,
-        func: Callable,
+        func: Callable[..., Any],
         arguments: Tuple[Any, ...]
 ) -> str:
     return output.decode()
@@ -96,15 +107,15 @@ def _convert_bytes_to_str(
 
 
 class Signature(NamedTuple):
-    return_type: Type = RETURN_CODE
-    argument_types: Sequence[Type] = ()
+    return_type: type[Any] = RETURN_CODE
+    argument_types: Sequence[type[Any]] = ()
 
 
 class DllWrapperMeta(type):
     """DLL-path-based 'singleton' metaclass for DLL wrapper classes"""
 
     # Only allow a single instance per DLL path.
-    _instances: WeakValueDictionary = WeakValueDictionary()  # of [str, Any]
+    _instances: WeakValueDictionary[str, Any] = WeakValueDictionary()
 
     # Note: without the 'type: ignore' for the ``__call__`` method below, mypy
     # generates 'Signature of "__call__" incompatible with supertype "type"'
@@ -164,7 +175,6 @@ class WrappedDll(metaclass=DllWrapperMeta):
     functions of interest in a subclass.
     """
 
-
     # This is the DLL library instance.
     _dll: ctypes.CDLL
 
