@@ -1,15 +1,21 @@
-from functools import partial
 import logging
-from typing import Sequence, Dict, Optional
+import time
+from functools import partial
+from typing import Any, Dict, List, Optional, Sequence, Union
 
 import numpy as np
 
 from qcodes.instrument.base import Instrument, InstrumentBase
-from qcodes.utils.validators import Numbers, Arrays, Strings, ComplexNumbers
-from qcodes.instrument.parameter import MultiParameter, Parameter, \
-    ArrayParameter, ParameterWithSetpoints
-from qcodes.instrument.channel import InstrumentChannel, ChannelList
-import random
+from qcodes.instrument.channel import ChannelList, InstrumentChannel
+from qcodes.instrument.parameter import (
+    ArrayParameter,
+    MultiParameter,
+    Parameter,
+    ParameterWithSetpoints,
+)
+from qcodes.utils.validators import Arrays, ComplexNumbers, Numbers, OnOff
+from qcodes.utils.validators import Sequence as ValidatorSequence
+from qcodes.utils.validators import Strings
 
 log = logging.getLogger(__name__)
 
@@ -104,7 +110,7 @@ class MockMetaParabola(InstrumentBase):
 class DummyInstrument(Instrument):
 
     def __init__(self, name: str = 'dummy',
-                 gates: Sequence = ('dac1', 'dac2', 'dac3'), **kwargs):
+                 gates: Sequence[str] = ('dac1', 'dac2', 'dac3'), **kwargs):
 
         """
         Create a dummy instrument that can be used for testing
@@ -118,13 +124,66 @@ class DummyInstrument(Instrument):
 
         # make gates
         for _, g in enumerate(gates):
-            self.add_parameter(g,
-                               parameter_class=Parameter,
-                               initial_value=0,
-                               label=f'Gate {g}',
-                               unit="V",
-                               vals=Numbers(-800, 400),
-                               get_cmd=None, set_cmd=None)
+            self.add_parameter(
+                g,
+                parameter_class=Parameter,
+                initial_value=0,
+                label=f"Gate {g}",
+                unit="V",
+                vals=Numbers(-800, 400),
+                get_cmd=None,
+                set_cmd=None,
+            )
+
+
+class DummyFailingInstrument(Instrument):
+    def __init__(self, name: str = "dummy", fail: bool = True, **kwargs):
+
+        """
+        Create a dummy instrument that fails on initialization
+        that can be used for testing
+
+        Args:
+            name: name for the instrument
+            fail: if true, instrument will throw a runtime error on creation.
+        """
+        super().__init__(name, **kwargs)
+
+        if fail:
+            raise RuntimeError("Failed to create instrument")
+
+
+class DummyAttrInstrument(Instrument):
+    def __init__(self, name: str = "dummy", **kwargs: Any):
+
+        """
+        Create a dummy instrument that can be used for testing.
+        This instrument has its parameters declared as attributes
+        and does not use add_parameter.
+        """
+        super().__init__(name, **kwargs)
+
+        self.ch1 = Parameter(
+            "ch1",
+            instrument=self,
+            initial_value=0,
+            label="Gate ch1",
+            unit="V",
+            vals=Numbers(-800, 400),
+            get_cmd=None,
+            set_cmd=None,
+        )
+
+        self.ch2 = Parameter(
+            "ch2",
+            instrument=self,
+            initial_value=0,
+            label="Gate ch2",
+            unit="V",
+            vals=Numbers(-800, 400),
+            get_cmd=None,
+            set_cmd=None,
+        )
 
 
 class DmmExponentialParameter(Parameter):
@@ -256,18 +315,24 @@ class DummyChannel(InstrumentChannel):
                            get_cmd=None,
                            set_cmd=None)
 
-        self.add_parameter('dummy_stop',
-                           unit='some unit',
-                           label='f stop',
-                           vals=Numbers(1, 1e3),
-                           get_cmd=None,
-                           set_cmd=None)
+        self.add_parameter(
+            "dummy_stop",
+            initial_value=100,
+            unit="some unit",
+            label="f stop",
+            vals=Numbers(1, 1e3),
+            get_cmd=None,
+            set_cmd=None,
+        )
 
-        self.add_parameter('dummy_n_points',
-                           unit='',
-                           vals=Numbers(1, 1e3),
-                           get_cmd=None,
-                           set_cmd=None)
+        self.add_parameter(
+            "dummy_n_points",
+            initial_value=101,
+            unit="",
+            vals=Numbers(1, 1e3),
+            get_cmd=None,
+            set_cmd=None,
+        )
 
         self.add_parameter('dummy_start_2',
                            initial_value=0,
@@ -277,18 +342,24 @@ class DummyChannel(InstrumentChannel):
                            get_cmd=None,
                            set_cmd=None)
 
-        self.add_parameter('dummy_stop_2',
-                           unit='some unit',
-                           label='f stop',
-                           vals=Numbers(1, 1e3),
-                           get_cmd=None,
-                           set_cmd=None)
+        self.add_parameter(
+            "dummy_stop_2",
+            initial_value=100,
+            unit="some unit",
+            label="f stop",
+            vals=Numbers(1, 1e3),
+            get_cmd=None,
+            set_cmd=None,
+        )
 
-        self.add_parameter('dummy_n_points_2',
-                           unit='',
-                           vals=Numbers(1, 1e3),
-                           get_cmd=None,
-                           set_cmd=None)
+        self.add_parameter(
+            "dummy_n_points_2",
+            initial_value=101,
+            unit="",
+            vals=Numbers(1, 1e3),
+            get_cmd=None,
+            set_cmd=None,
+        )
 
         self.add_parameter('dummy_sp_axis',
                            unit='some unit',
@@ -361,7 +432,7 @@ class DummyChannelInstrument(Instrument):
             channel = DummyChannel(self, f'Chan{chan_name}', chan_name)
             channels.append(channel)
             self.add_submodule(chan_name, channel)
-        self.add_submodule("channels", channels)
+        self.add_submodule("channels", channels.to_channel_tuple())
 
 
 class MultiGetter(MultiParameter):
@@ -394,24 +465,33 @@ class MultiSetPointParam(MultiParameter):
     Multiparameter which only purpose it to test that units, setpoints
     and so on are copied correctly to the individual arrays in the datarray.
     """
-    def __init__(self, instrument=None, name='multi_setpoint_param'):
+
+    def __init__(self, instrument=None, name="multi_setpoint_param", **kwargs):
         shapes = ((5,), (5,))
         names = ('multi_setpoint_param_this', 'multi_setpoint_param_that')
         labels = ('this label', 'that label')
         units = ('this unit', 'that unit')
         sp_base = tuple(np.linspace(5, 9, 5))
         setpoints = ((sp_base,), (sp_base,))
-        setpoint_names = (('multi_setpoint_param_this_setpoint',), ('multi_setpoint_param_this_setpoint',))
-        setpoint_labels = (('this setpoint',), ('this setpoint',))
-        setpoint_units = (('this setpointunit',), ('this setpointunit',))
-        super().__init__(name, names, shapes,
-                         instrument=instrument,
-                         labels=labels,
-                         units=units,
-                         setpoints=setpoints,
-                         setpoint_labels=setpoint_labels,
-                         setpoint_names=setpoint_names,
-                         setpoint_units=setpoint_units)
+        setpoint_names = (
+            ("multi_setpoint_param_this_setpoint",),
+            ("multi_setpoint_param_this_setpoint",),
+        )
+        setpoint_labels = (("this setpoint",), ("this setpoint",))
+        setpoint_units = (("this setpointunit",), ("this setpointunit",))
+        super().__init__(
+            name,
+            names,
+            shapes,
+            instrument=instrument,
+            labels=labels,
+            units=units,
+            setpoints=setpoints,
+            setpoint_labels=setpoint_labels,
+            setpoint_names=setpoint_names,
+            setpoint_units=setpoint_units,
+            **kwargs,
+        )
 
     def get_raw(self):
         items = (np.zeros(5), np.ones(5))
@@ -424,7 +504,7 @@ class Multi2DSetPointParam(MultiParameter):
     and so on are copied correctly to the individual arrays in the datarray.
     """
 
-    def __init__(self, instrument=None, name='multi_2d_setpoint_param'):
+    def __init__(self, instrument=None, name="multi_2d_setpoint_param", **kwargs):
         shapes = ((5, 3), (5, 3))
         names = ('this', 'that')
         labels = ('this label', 'that label')
@@ -433,22 +513,37 @@ class Multi2DSetPointParam(MultiParameter):
         sp_base_2 = tuple(np.linspace(9, 11, 3))
         array_setpoints = setpoint_generator(sp_base_1, sp_base_2)
         setpoints = (array_setpoints, array_setpoints)
-        setpoint_names = (('multi_2d_setpoint_param_this_setpoint', 'multi_2d_setpoint_param_that_setpoint'),
-                          ('multi_2d_setpoint_param_this_setpoint', 'multi_2d_setpoint_param_that_setpoint'))
-        setpoint_labels = (('this setpoint', 'that setpoint'),
-                           ('this setpoint', 'that setpoint'))
-        setpoint_units = (('this setpointunit',
-                           'that setpointunit'),
-                          ('this setpointunit',
-                           'that setpointunit'))
-        super().__init__(name, names, shapes,
-                         instrument=instrument,
-                         labels=labels,
-                         units=units,
-                         setpoints=setpoints,
-                         setpoint_labels=setpoint_labels,
-                         setpoint_names=setpoint_names,
-                         setpoint_units=setpoint_units)
+        setpoint_names = (
+            (
+                "multi_2d_setpoint_param_this_setpoint",
+                "multi_2d_setpoint_param_that_setpoint",
+            ),
+            (
+                "multi_2d_setpoint_param_this_setpoint",
+                "multi_2d_setpoint_param_that_setpoint",
+            ),
+        )
+        setpoint_labels = (
+            ("this setpoint", "that setpoint"),
+            ("this setpoint", "that setpoint"),
+        )
+        setpoint_units = (
+            ("this setpointunit", "that setpointunit"),
+            ("this setpointunit", "that setpointunit"),
+        )
+        super().__init__(
+            name,
+            names,
+            shapes,
+            instrument=instrument,
+            labels=labels,
+            units=units,
+            setpoints=setpoints,
+            setpoint_labels=setpoint_labels,
+            setpoint_names=setpoint_names,
+            setpoint_units=setpoint_units,
+            **kwargs,
+        )
 
     def get_raw(self):
         items = (np.zeros((5, 3)), np.ones((5, 3)))
@@ -462,7 +557,7 @@ class Multi2DSetPointParam2Sizes(MultiParameter):
     shapes.
     """
 
-    def __init__(self, instrument=None, name='multi_2d_setpoint_param'):
+    def __init__(self, instrument=None, name="multi_2d_setpoint_param", **kwargs):
         shapes = ((5, 3), (2, 7))
         names = ('this_5_3', 'this_2_7')
         labels = ('this label', 'that label')
@@ -474,22 +569,37 @@ class Multi2DSetPointParam2Sizes(MultiParameter):
         sp_base_2_2 = tuple(np.linspace(9, 11, 7))
         array_setpoints_2 = setpoint_generator(sp_base_1_2, sp_base_2_2)
         setpoints = (array_setpoints_1, array_setpoints_2)
-        setpoint_names = (('multi_2d_setpoint_param_this_setpoint_1', 'multi_2d_setpoint_param_that_setpoint_1'),
-                          ('multi_2d_setpoint_param_this_setpoint_2', 'multi_2d_setpoint_param_that_setpoint_2'))
-        setpoint_labels = (('this setpoint 1', 'that setpoint 1'),
-                           ('this setpoint 2', 'that setpoint 2'))
-        setpoint_units = (('this setpointunit',
-                           'that setpointunit'),
-                          ('this setpointunit',
-                           'that setpointunit'))
-        super().__init__(name, names, shapes,
-                         instrument=instrument,
-                         labels=labels,
-                         units=units,
-                         setpoints=setpoints,
-                         setpoint_labels=setpoint_labels,
-                         setpoint_names=setpoint_names,
-                         setpoint_units=setpoint_units)
+        setpoint_names = (
+            (
+                "multi_2d_setpoint_param_this_setpoint_1",
+                "multi_2d_setpoint_param_that_setpoint_1",
+            ),
+            (
+                "multi_2d_setpoint_param_this_setpoint_2",
+                "multi_2d_setpoint_param_that_setpoint_2",
+            ),
+        )
+        setpoint_labels = (
+            ("this setpoint 1", "that setpoint 1"),
+            ("this setpoint 2", "that setpoint 2"),
+        )
+        setpoint_units = (
+            ("this setpointunit", "that setpointunit"),
+            ("this setpointunit", "that setpointunit"),
+        )
+        super().__init__(
+            name,
+            names,
+            shapes,
+            instrument=instrument,
+            labels=labels,
+            units=units,
+            setpoints=setpoints,
+            setpoint_labels=setpoint_labels,
+            setpoint_names=setpoint_names,
+            setpoint_units=setpoint_units,
+            **kwargs,
+        )
 
     def get_raw(self):
         items = (np.zeros((5, 3)), np.ones((2, 7)))
@@ -501,17 +611,23 @@ class MultiScalarParam(MultiParameter):
     Multiparameter whos elements are scalars i.e. similar to
     Parameter with no setpoints etc.
     """
-    def __init__(self, instrument=None, name='multiscalarparameter'):
+
+    def __init__(self, instrument=None, name="multiscalarparameter", **kwargs):
         shapes = ((), ())
         names = ('thisparam', 'thatparam')
         labels = ('thisparam label', 'thatparam label')
         units = ('thisparam unit', 'thatparam unit')
         setpoints = ((), ())
-        super().__init__(name, names, shapes,
-                         instrument=instrument,
-                         labels=labels,
-                         units=units,
-                         setpoints=setpoints)
+        super().__init__(
+            name,
+            names,
+            shapes,
+            instrument=instrument,
+            labels=labels,
+            units=units,
+            setpoints=setpoints,
+            **kwargs,
+        )
 
     def get_raw(self):
         items = (0, 1)
@@ -524,24 +640,27 @@ class ArraySetPointParam(ArrayParameter):
     and so on are copied correctly to the individual arrays in the datarray.
     """
 
-    def __init__(self, instrument=None, name='array_setpoint_param'):
+    def __init__(self, instrument=None, name="array_setpoint_param", **kwargs):
         shape = (5,)
         label = 'this label'
         unit = 'this unit'
         sp_base = tuple(np.linspace(5, 9, 5))
         setpoints = (sp_base,)
-        setpoint_names = ('array_setpoint_param_this_setpoint',)
-        setpoint_labels = ('this setpoint',)
-        setpoint_units = ('this setpointunit',)
-        super().__init__(name,
-                         shape,
-                         instrument,
-                         label=label,
-                         unit=unit,
-                         setpoints=setpoints,
-                         setpoint_labels=setpoint_labels,
-                         setpoint_names=setpoint_names,
-                         setpoint_units=setpoint_units)
+        setpoint_names = ("array_setpoint_param_this_setpoint",)
+        setpoint_labels = ("this setpoint",)
+        setpoint_units = ("this setpointunit",)
+        super().__init__(
+            name,
+            shape,
+            instrument,
+            label=label,
+            unit=unit,
+            setpoints=setpoints,
+            setpoint_labels=setpoint_labels,
+            setpoint_names=setpoint_names,
+            setpoint_units=setpoint_units,
+            **kwargs,
+        )
 
     def get_raw(self):
         item = np.ones(5) + 1
@@ -553,24 +672,27 @@ class ComplexArraySetPointParam(ArrayParameter):
     Arrayparameter that returns complex numbers
     """
 
-    def __init__(self, instrument=None, name='testparameter'):
+    def __init__(self, instrument=None, name="testparameter", **kwargs):
         shape = (5,)
         label = 'this label'
         unit = 'this unit'
         sp_base = tuple(np.linspace(5, 9, 5))
         setpoints = (sp_base,)
-        setpoint_names = ('this_setpoint',)
-        setpoint_labels = ('this setpoint',)
-        setpoint_units = ('this setpointunit',)
-        super().__init__(name,
-                         shape,
-                         instrument,
-                         label=label,
-                         unit=unit,
-                         setpoints=setpoints,
-                         setpoint_labels=setpoint_labels,
-                         setpoint_names=setpoint_names,
-                         setpoint_units=setpoint_units)
+        setpoint_names = ("this_setpoint",)
+        setpoint_labels = ("this setpoint",)
+        setpoint_units = ("this setpointunit",)
+        super().__init__(
+            name,
+            shape,
+            instrument,
+            label=label,
+            unit=unit,
+            setpoints=setpoints,
+            setpoint_labels=setpoint_labels,
+            setpoint_names=setpoint_names,
+            setpoint_units=setpoint_units,
+            **kwargs,
+        )
 
     def get_raw(self):
         item = np.arange(5) - 1j*np.arange(5)
@@ -692,9 +814,255 @@ class SnapShotTestInstrument(Instrument):
 
     def snapshot_base(self, update: Optional[bool] = True,
                       params_to_skip_update: Optional[Sequence[str]] = None
-                      ) -> Dict:
+                      ) -> Dict[Any, Any]:
         if params_to_skip_update is None:
             params_to_skip_update = self._params_to_skip
         snap = super().snapshot_base(
             update=update, params_to_skip_update=params_to_skip_update)
         return snap
+
+
+class MockField(Instrument):
+
+    def __init__(
+            self,
+            name: str,
+            vals: Numbers = Numbers(min_value=-1., max_value=1.),
+            **kwargs):
+        """Mock instrument for emulating a magnetic field axis
+
+        Args:
+            name (str): Instrument name
+            vals (Numbers, optional): Soft limits. Defaults to Numbers(min_value=-1., max_value=1.).
+        """
+        super().__init__(name=name, **kwargs)
+        self._field = 0.0
+        self.add_parameter("field",
+                           parameter_class=Parameter,
+                           initial_value=0.0,
+                           unit='T',
+                           vals=vals,
+                           get_cmd=self.get_field, set_cmd=self.set_field)
+        self.add_parameter("ramp_rate",
+                           parameter_class=Parameter,
+                           initial_value=0.1,
+                           unit='T/min',
+                           get_cmd=None, set_cmd=None)
+        self._ramp_start_time: Optional[float] = None
+        self._wait_time: Optional[float] = None
+        self._fr = self._field_ramp()
+        next(self._fr)
+
+    def get_field(self):
+        """
+        This method is automatically wrapped to
+        provide a ``get`` method on the parameter instance.
+        """
+        if self._ramp_start_time:
+            _time_since_start = time.time() - self._ramp_start_time
+            val = self._fr.send(_time_since_start)
+            next(self._fr)
+            self._field = val
+        return self._field
+
+    def set_field(self, value, block: bool = True):
+        if self._field == value:
+            return value
+
+        wait_time = 60. * np.abs(self._field - value) / self.ramp_rate()
+        self._wait_time = wait_time
+        self._sign = np.sign(value - self._field)
+        self._start_field = self._field
+        self._target_field = value
+        self._ramp_start_time = time.time()
+
+        if block:
+            time.sleep(wait_time)
+            self._field = value
+            return value
+
+    def _field_ramp_fcn(self, _time: float):
+        if self._wait_time is None:
+            return self._field
+        elif _time <= 0.0:
+            return self._start_field
+        elif _time >= self._wait_time:
+            return self._target_field
+        dfield = self.ramp_rate() * _time / 60.0
+        return self._start_field + self._sign * dfield
+
+    def _field_ramp(self):
+        """
+        Yields field for a given point in time
+        """
+        while True:
+            _time = yield
+            if _time is None:
+                _time = 0.0
+
+            yield float(self._field_ramp_fcn(_time))
+
+
+class MockLockin(Instrument):
+
+    def __init__(
+            self,
+            name: str,
+            **kwargs):
+        super().__init__(name=name, **kwargs)
+        self.add_parameter("X",
+                           parameter_class=Parameter,
+                           initial_value=1e-3,
+                           unit='V',
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter("Y",
+                           parameter_class=Parameter,
+                           initial_value=1e-5,
+                           unit='V',
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter("frequency",
+                           parameter_class=Parameter,
+                           initial_value=125.,
+                           unit='Hz',
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter("amplitude",
+                           parameter_class=Parameter,
+                           initial_value=0.,
+                           unit='V',
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter("phase",
+                           parameter_class=Parameter,
+                           initial_value=0.,
+                           unit='deg',
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter("time_constant",
+                           parameter_class=Parameter,
+                           initial_value=1.e-3,
+                           unit='s',
+                           get_cmd=None, set_cmd=None)
+
+
+class MockDACChannel(InstrumentChannel):
+    """
+    A single dummy channel implementation
+    """
+
+    def __init__(self, parent, name, num):
+        super().__init__(parent, name)
+
+        self._num = num
+        self.add_parameter('voltage',
+                           parameter_class=Parameter,
+                           initial_value=0.,
+                           label=f"Voltage_{name}",
+                           unit='V',
+                           vals=Numbers(-2., 2.),
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter('dac_output',
+                           parameter_class=Parameter,
+                           initial_value="off",
+                           vals=OnOff(),
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter('smc',
+                           parameter_class=Parameter,
+                           initial_value="off",
+                           vals=OnOff(),
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter('bus',
+                           parameter_class=Parameter,
+                           initial_value="off",
+                           vals=OnOff(),
+                           get_cmd=None, set_cmd=None)
+        self.add_parameter('gnd',
+                           parameter_class=Parameter,
+                           initial_value="off",
+                           vals=OnOff(),
+                           get_cmd=None, set_cmd=None)
+
+    def channel_number(self):
+        return self._num
+
+
+class MockDAC(Instrument):
+
+    def __init__(
+        self,
+        name: str = 'mdac',
+        num_channels: int = 10,
+        **kwargs):
+
+        """
+        Create a dummy instrument that can be used for testing
+
+        Args:
+            name: name for the instrument
+            gates: list of names that is used to create parameters for
+                            the instrument
+        """
+        super().__init__(name, **kwargs)
+
+        # make gates
+        channels = ChannelList(self, "channels", MockDACChannel)
+        for n in range(num_channels):
+            num = str(n + 1).zfill(2)
+            chan_name = f"ch{num}"
+            channel = MockDACChannel(parent=self, name=chan_name, num=num)
+            channels.append(channel)
+            self.add_submodule(chan_name, channel)
+        self.add_submodule("channels", channels.to_channel_tuple())
+
+
+class MockCustomChannel(InstrumentChannel):
+    def __init__(
+        self,
+        parent: InstrumentBase,
+        name: str,
+        channel: Union[str, InstrumentChannel],
+        current_valid_range: Optional[List[float]] = None,
+    ) -> None:
+        """
+        A custom instrument channel emulating an existing channel.
+
+        It adds a parameter not found in the original channel, the
+        current_valid_range.
+        Args:
+            parent: Instrument to which the original channel belongs to,
+                usually a dac.
+            name: Name of channel.
+            channel: The original instrument channel.
+            current_valid_range: Voltage range the channel is expected to show
+                interesting features. It's just an example of an additional
+                parameter a regular instrument channel does not have.
+        """
+        if isinstance(channel, str):
+            _, channel_name = channel.split(".")
+            instr_channel = getattr(parent, channel_name)
+            self._dac_channel = instr_channel
+        elif isinstance(channel, InstrumentChannel):
+            self._dac_channel = channel
+        else:
+            raise ValueError('Unknown input type for "channel".')
+
+        super().__init__(parent, name)
+
+        if current_valid_range is None:
+            current_valid_range = []
+        super().add_parameter(
+            name="current_valid_range",
+            label=f"{name} valid voltage range",
+            initial_value=current_valid_range,
+            vals=ValidatorSequence(Numbers(), length=2),
+            get_cmd=None,
+            set_cmd=None,
+        )
+
+        self.add_parameter(
+            "voltage",
+            parameter_class=Parameter,
+            initial_value=0.0,
+            label=f"Voltage_{name}",
+            unit="V",
+            vals=Numbers(-2.0, 2.0),
+            get_cmd=None,
+            set_cmd=None,
+        )
