@@ -1,9 +1,12 @@
 from collections import namedtuple
+from typing import Any, Optional
 
 import pytest
 
+from qcodes.instrument.base import InstrumentBase
 from qcodes.instrument.parameter import Parameter
 import qcodes.utils.validators as vals
+from qcodes.tests.instrument_mocks import DummyChannelInstrument
 
 NOT_PASSED = 'NOT_PASSED'
 
@@ -38,6 +41,13 @@ def cache_is_valid(request):
     return request.param
 
 
+@pytest.fixture(name="dummy_instrument")
+def _make_dummy_instrument():
+    instr = DummyChannelInstrument("dummy")
+    yield instr
+    instr.close()
+
+
 class GettableParam(Parameter):
     """ Parameter that keeps track of number of get operations"""
     def __init__(self, *args, **kwargs):
@@ -109,7 +119,7 @@ class GetSetRawParameter(Parameter):
         pass
 
 
-class BookkeepingValidator(vals.Validator):
+class BookkeepingValidator(vals.Validator[Any]):
     """
     Validator that keeps track of what it validates
     """
@@ -143,6 +153,19 @@ class MemoryParameter(Parameter):
         return get_func
 
 
+class VirtualParameter(Parameter):
+    def __init__(self, name: str, param: Parameter, **kwargs):
+        self._param = param
+        super().__init__(name=name, **kwargs)
+
+    @property
+    def underlying_instrument(self) -> Optional[InstrumentBase]:
+        return self._param.instrument
+
+    def get_raw(self):
+        return self._param.get()
+
+
 blank_instruments = (
     None,  # no instrument at all
     namedtuple('noname', '')(),  # no .name
