@@ -6,6 +6,7 @@ import math
 import numbers
 import os
 import time
+import warnings
 from asyncio import iscoroutinefunction
 from collections import OrderedDict, abc
 from contextlib import contextmanager
@@ -13,9 +14,25 @@ from copy import deepcopy
 from functools import partial
 from inspect import signature
 from pathlib import Path
-from typing import (TYPE_CHECKING, Any, Callable, Dict, Iterator, List,
-                    Mapping, MutableMapping, Optional, Sequence, SupportsAbs,
-                    Tuple, Type, Union, cast, Hashable, TypeVar)
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Hashable,
+    Iterator,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Sequence,
+    SupportsAbs,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 import numpy as np
 
@@ -41,23 +58,34 @@ class NumpyJSONEncoder(json.JSONEncoder):
     def default(self, obj: Any) -> Any:
         """
         List of conversions that this encoder performs:
+
         * ``numpy.generic`` (all integer, floating, and other types) gets
-        converted to its python equivalent using its ``item`` method (see
-        ``numpy`` docs for more information,
-        https://docs.scipy.org/doc/numpy/reference/arrays.scalars.html).
+          converted to its python equivalent using its ``item`` method (see
+          ``numpy`` docs for more information,
+          https://docs.scipy.org/doc/numpy/reference/arrays.scalars.html).
         * ``numpy.ndarray`` gets converted to python list using its ``tolist``
-        method.
+          method.
         * Complex number (a number that conforms to ``numbers.Complex`` ABC) gets
-        converted to a dictionary with fields ``re`` and ``im`` containing floating
-        numbers for the real and imaginary parts respectively, and a field
-        ``__dtype__`` containing value ``complex``.
+          converted to a dictionary with fields ``re`` and ``im`` containing floating
+          numbers for the real and imaginary parts respectively, and a field
+          ``__dtype__`` containing value ``complex``.
+        * Numbers with uncertainties  (numbers that conforms to ``uncertainties.UFloat``) get
+          converted to a dictionary with fields ``nominal_value`` and ``std_dev`` containing floating
+          numbers for the nominal and uncertainty parts respectively, and a field
+          ``__dtype__`` containing value ``UFloat``.
         * Object with a ``_JSONEncoder`` method get converted the return value of
-        that method.
+          that method.
         * Objects which support the pickle protocol get converted using the
-        data provided by that protocol.
+          data provided by that protocol.
         * Other objects which cannot be serialized get converted to their
-        string representation (suing the ``str`` function).
+          string representation (using the ``str`` function).
         """
+        with warnings.catch_warnings():
+            # this context manager can be removed when uncertainties
+            # no longer triggers deprecation warnings
+            warnings.simplefilter("ignore", category=DeprecationWarning)
+            import uncertainties
+
         if isinstance(obj, np.generic) \
                 and not isinstance(obj, np.complexfloating):
             # for numpy scalars
@@ -72,6 +100,12 @@ class NumpyJSONEncoder(json.JSONEncoder):
                 're': float(obj.real),
                 'im': float(obj.imag)
             }
+        elif isinstance(obj, uncertainties.UFloat):
+            return {
+                '__dtype__': 'UFloat',
+                'nominal_value': float(obj.nominal_value),
+                'std_dev': float(obj.std_dev)
+            }
         elif hasattr(obj, '_JSONEncoder'):
             # Use object's custom JSON encoder
             jsosencode = getattr(obj, "_JSONEncoder")
@@ -166,8 +200,7 @@ def is_sequence_of(obj: Any,
     return True
 
 
-def is_function(f: Callable[..., Any],
-                arg_count: int, coroutine: bool = False) -> bool:
+def is_function(f: object, arg_count: int, coroutine: bool = False) -> bool:
     """
     Check and require a function that can accept the specified number of
     positional arguments, which either is or is not a coroutine
@@ -576,8 +609,7 @@ def add_to_spyder_UMR_excludelist(modulename: str) -> None:
             sitecustomize_found = True
         if sitecustomize_found is False:
             try:
-                from spyder_kernels.customize import \
-                    spydercustomize as sitecustomize
+                from spyder_kernels.customize import spydercustomize as sitecustomize
 
             except ImportError:
                 pass
@@ -623,7 +655,7 @@ def partial_with_docstring(func: Callable[..., Any],
                            docstring: str,
                            **kwargs: Any) -> Callable[..., Any]:
     """
-    We want to have a partial function which will allow us access the docstring
+    We want to have a partial function which will allow us to access the docstring
     through the python built-in help function. This is particularly important
     for client-facing driver methods, whose arguments might not be obvious.
 
@@ -634,7 +666,7 @@ def partial_with_docstring(func: Callable[..., Any],
     >>> ... pass
     >>> g = partial(f)
     >>> g.__doc__ = "bla"
-    >>> help(g) # this will print an unhelpful message
+    >>> help(g) # this will print the docstring of partial and not the docstring set above
 
     Args:
         func: A function that its docstring will be accessed.
@@ -642,8 +674,8 @@ def partial_with_docstring(func: Callable[..., Any],
     """
     ex = partial(func, **kwargs)
 
-    def inner(**inner_kwargs: Any) -> None:
-        ex(**inner_kwargs)
+    def inner(*inner_args: Any, **inner_kwargs: Any) -> Any:
+        return ex(*inner_args, **inner_kwargs)
 
     inner.__doc__ = docstring
 
